/* Enhanced Vote for Waffle				       	           */
/* Supplemental Recent Topic Command					   */

/* Version 2.0   							   */
/* By Christopher Horn (chorn@eastwind.mcds.com)        		   */
/* Compiled 7/30/92 under Borland C/C++ 3.0				   */
/* This code is public domain. Modify to your hearts content.		   */

/* Inspired by the original vote beast by Victor Spicer, and several       */
/* recent mods, most notably by Mike Wallis (mwallis@clubzen.fidonet.org)  */
/* and Kevin Day (kday@entropy.mcds.com)				   */


#include<stdio.h>
#include<stdlib.h>
#include<string.h>
			/* Defines for general ASCII characters            */
#define VTCLR  "\033[2J"
#define TTCLR  "\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n"

			/* Global variables				   */

char topic[70];			        /* The last stale topic            */
int tin;                                /* The stale topics number         */

char comment[16][70];			/* To store comments	           */
int cin[16];				/* Total votes for the comments    */
int topcom;				/* Number of comments		   */

int people;				/* Total people in user file	   */
int votes;				/* People who actually voted	   */

char user[8];				/* Account name			   */
char uname[30];				/* Users identity                  */

FILE *fp;                               /* Our file pointer		   */
int i,j,k;				/* Loop control variables	   */
char tmp[70];				/* Temporary string read variable  */
char fname[100];			/* File name			   */
char wk[10];				/* Work string  		   */
char ttype[8];				/* Hold term type		   */
char clr[26];				/* Clear sequence		   */

char fpath[80] = "/waffle/vote/";       /* For path to vote directory	   */

			/* Function prototypes				   */

void rdata(void);			/* Read in recent topic data	   */
void dispscrn(char *st);		        /* Display welcome screen          */
void dcomments(void);		        /* Display last comments	   */
void getcnf(void);			/* Get configuration info	   */
void frerr(void);			/* Report open for read error	   */


/*-------------------------------------------------------------------------*/

int main(int argc, char *argv[])
{
  if (argc<4)		/* Someone forgot to call with the arguements	   */
  {
    printf("\nEnhanced Vote for Waffle\nSupplemental Recent Topic Command\n\n"
	   "Version 2.0\nBy Christopher Horn\n\nError : You must pass the "
	   "account name, term type, and user\n        identity as"
	   " arguments\n\n");
    exit(0);
  }
  else			/* Read in arguements				   */
  {
    strcpy(user,argv[1]);               /* Get account name		   */
    strcpy(ttype,argv[2]);		/* Get terminal type		   */
    if(!strcmp(ttype,"none"))
      strcpy(clr,TTCLR);                /* Set clear sequence		   */
    else
      strcpy(clr,VTCLR);
    strcpy(uname,argv[3]);
    if(argc > 4)                        /* Get users identity. Set up so   */
    {                                   /* can handle middle and last name */
      strcat(uname," ");                /* if they are passed.             */
      strcat(uname,argv[4]);
    }
    if(argc > 5)
    {
      strcat(uname," ");
      strcat(uname,argv[5]);
    }
  }

  getcnf();				/* Get config info needed          */
  rdata();				/* Get recent topic data	   */
  dispscrn("REPORT");		      	/* Display stats screen		   */
  return 0;
}


/*-------------------------------------------------------------------------*/
/* rdata  Read in the recent topic data. If file does not exist, let user  */
/* know no topics have turned stale yet.				   */

void rdata(void)
{
  strcpy(fname,fpath);                  /* Get topic info first            */
  strcat(fname,"recent");
  if ((fp = fopen(fname,"r")) == NULL)
  {
    dispscrn("NOREPORT");
    exit(0);
  }
  fscanf(fp,"%d %[^\n]",&tin,topic);
  fclose(fp);

  strcpy(fname,fpath);                  /* Now comment info                */
  strcat(fname,"v.");
  itoa(tin,wk,10);
  strcat(fname,wk);
  if ((fp = fopen(fname,"r")) == NULL) frerr();
  fscanf(fp,"%d",&topcom);
  for(i=0; i<=topcom; i++) fscanf(fp,"%d %[^\n]",&cin[i],comment[i]);
  fclose(fp);

  strcpy(fname,fpath);                  /* Now find out how many people    */
  strcat(fname,"u.");                   /* really voted on something.      */
  strcat(fname,wk);
  if ((fp = fopen(fname,"r")) == NULL) frerr();
  votes = 0;
  fscanf(fp,"%d",&people);
  for(i=0; i<=people; i++)
  {
    fscanf(fp,"%d %s",&j,tmp);
    if(j != 0) votes++;
  }
  fclose(fp);
}


/*-------------------------------------------------------------------------*/
/* dispscrn  Display screen contained in the passed filename. The file 	   */
/* should exist under the vote directory.				   */

void dispscrn(char *st)
{
  strcpy(fname,fpath);
  strcat(fname,st);
  if ((fp = fopen(fname,"r")) == NULL) frerr();
  while((i = getc(fp))!=EOF)
  {
    switch(i)
    {                                   /* Some support for Waffle codes   */
      case 12:                          /* Recognize ctrl-l (pause for a   */
	while(getch()!=13);             /* return), ~* (clear screen),     */
	break;                          /* ~%A (account name), ~%W (user   */
      case 126:                         /* identity), ~%t (term type).     */
	j = getc(fp);
	if (j == 42) printf("%s",clr);
	if (j == 37)                    /* Custom codes are ~%T (topic),   */
	{                               /* ~%C (comments), and of course   */
	  j = getc(fp);                 /* ~%P (number people voted)       */
	  if (j == 65) printf("%s",user);
	  if (j == 87) printf("%s",uname);
	  if (j == 116) printf("%s",ttype);
	  if (j == 84) printf("%s",topic);
	  if (j == 80) printf("%d",votes);
	  if (j == 67) dcomments();
	}
	break;
      default:
	putchar(i);
    }
  }
  fclose(fp);
}


/*-------------------------------------------------------------------------*/
/* dcomments  Display comments						   */

void dcomments(void)
{
  for(i=0; i<=topcom; i++)
  {
    printf("%2d  %s",i+1,comment[i]);
    for (j=1; j<66-strlen(comment[i]); j++) printf(".");
    printf("%3d\n",cin[i]);
  }
}


/*-------------------------------------------------------------------------*/
/* getcnf  Get configuration info if a config file exists		   */

void getcnf(void)
{
  if((fp = fopen("\\waffle\\system\\vote.cfg","r")) != NULL)
  {
    while(!feof(fp))
    {
      fscanf(fp,"%[^\n]",tmp);
      if(getc(fp)==EOF) break;
      strupr(tmp);
      if(!strncmp("PATH=",tmp,5))       /* All we need here is the path	   */
      {
	strcpy(fpath,tmp+5);
	break;
      }
    }
    fclose(fp);
  }
}


/*-------------------------------------------------------------------------*/
/* frerr  Report a file open for read i/o error and abort program			   */

void frerr(void)
{
  printf("\n\n\nError : Unable to open %s for reading\n\n",fname);
  exit(0);
}

